{# Note - assumes django_jsform/django_jsform.css is in use #}

{% load jsform %}

{% for field in form %}
{% if field.is_hidden %}
	{{field}}
{% else %}
<div class="block tightlines">
	{# Single checkbox fields are handled differently - field comes before label text, inline #}
	{% if field|is_single_checkbox %}
		{% include "django_jsform/checkbox_field.html" %}
	{% else %}
		{% include "django_jsform/standard_field.html" %}
	{% endif %}
</div>
{% endif %}
{% endfor %}